"""
This module contains FlagObject implementation.

There is no need to import this module directly. Use the methods of
the L{Ruleset object <ruleset.Ruleset>} to create and manipulate
FlagObjects.

    >>> o = r.object_add("/FlagObjects/Test")
    >>> o.add("Interactive")
    >>> print o.items
    ['Interactive']

@var __revision__: Revision identifier.
"""

__revision__ = '%K%'

import plobject
from thisversion import PLDB
import plutil

class FlagObject(plobject.PLObject):
	"""
	FlagObject Implementation, for XFB (Transfer Behaviour) Flag.

	Object path: /FlagObjects

	@cvar Item: FlagObject Item implementation.
	"""

	__slots__ = ('_prop', '_pldb', '_callback', '_item_added')
	_may_exclude = True

	base_path = plutil.roConst("/FlagObjects", "Only the type portion of the path")
	type = plutil.roConst(PLDB.OBJECT_TYPE_FLAGOBJECT, "Number representing the type of object")

	def _create_item(self, value):
		if value not in self._pldb.xfb_flags_list():
			raise ValueError("Invalid flag (%s). See Ruleset.xfb_flags_list() for valid flags." % (value,))
		
		return self.Item(self, value=value)

	def add(self, value, exclude=False):
		"""
		Add a FlagObject Item to this FlagObject

		@type value: C{str}
		@param value: flag

		@type exclude: C{boolean}
		@param exclude: If this is set to True the flag in this item must be excluded,
			otherwise the flag in this item must be included.

		"""
		plobject.PLObject.add(self, value, exclude)

	class Item(plobject.PLObject.Item):
		"""
		FlagObject Item implementation.
		Should not be instantiated, use L{FlagObject.add} instead.

		@type value: C{str}
		"""
		__slots__ = ('_prop', '_parent')
		_Prop = plutil.Prop(exclude=bool, value=str, *plobject._item_props)

		def __str__(self):
			return "%s%s" % (['', '!'][self._prop.exclude], self._prop.value)
		
		value = property(lambda self: self._prop.value, doc="Flag")
		exclude = property(lambda self: self._prop.exclude, doc="True = exclude, False = include")
